/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdio.h>
#include <zephyr/kernel.h>

#include <nrfx_gpiote.h>
#include <helpers/nrfx_gppi.h>
#if defined(DPPI_PRESENT)
#include <nrfx_dppi.h>
#else
#include <nrfx_ppi.h>
#endif

#include <zephyr/logging/log.h>
#include <zephyr/irq.h>
#include <zephyr/drivers/gpio.h>
LOG_MODULE_REGISTER(nrfx_sample, LOG_LEVEL_INF);

#define GPIOTE_INST	NRF_DT_GPIOTE_INST(DT_NODELABEL(signal_input_pins), gpios)

#define INPUT_SIG_1 NRF_DT_GPIOS_TO_PSEL_BY_IDX(DT_NODELABEL(signal_input_pins), gpios, 0)
#define INPUT_SIG_2 NRF_DT_GPIOS_TO_PSEL_BY_IDX(DT_NODELABEL(signal_input_pins), gpios, 1)

const nrfx_gpiote_t gpiote = NRFX_GPIOTE_INSTANCE(GPIOTE_INST);

static void pin_evt_handler(nrfx_gpiote_pin_t pin,
			   nrfx_gpiote_trigger_t trigger,
			   void *context)
{
	LOG_INF("GPIO input event callback for P%d.%d", 
		NRF_PIN_NUMBER_TO_PORT(pin), NRF_PIN_NUMBER_TO_PIN(pin));
}

void gpiote_init(void) 
{
	int err; 
	static const nrf_gpio_pin_pull_t pull_config = NRF_GPIO_PIN_PULLUP;
	uint8_t in_sig1_channel, in_sig2_channel;

	static const nrfx_gpiote_handler_config_t handler_config = {
    	.handler = pin_evt_handler,
	};

	err = nrfx_gpiote_init(&gpiote, 0);
	if (err != NRFX_SUCCESS && err != NRFX_ERROR_ALREADY) {
		LOG_ERR("nrfx_gpiote_init error: 0x%08X", err);
		return;
	}

	// Configure input for SIGNAL 1
	nrfx_gpiote_trigger_config_t trigger_config_sig1 = {
	    .trigger = NRFX_GPIOTE_TRIGGER_HITOLO,
	    .p_in_channel = &in_sig1_channel,
	};
	nrfx_gpiote_input_pin_config_t input_config = {
	    .p_pull_config = &pull_config,
	    .p_trigger_config = &trigger_config_sig1,
	    .p_handler_config = &handler_config
	};	

	err = nrfx_gpiote_channel_alloc(&gpiote, &in_sig1_channel);
	if (err != NRFX_SUCCESS) {
		LOG_ERR("Failed to allocate in_channel, error: 0x%08X", err);
		return;
	}

	err = nrfx_gpiote_channel_alloc(&gpiote, &in_sig2_channel);
	if (err != NRFX_SUCCESS) {
		LOG_ERR("Failed to allocate in_channel, error: 0x%08X", err);
		return;
	}
	
	err = nrfx_gpiote_input_configure(&gpiote, INPUT_SIG_1, &input_config);
	if (err != NRFX_SUCCESS) {
		LOG_ERR("nrfx_gpiote_input_configure error: 0x%08X", err);
		return;
	}
	
	// Reconfigure trigger for SIGNAL 2
	nrfx_gpiote_trigger_config_t trigger_config_sig2 = {
	    .trigger = NRFX_GPIOTE_TRIGGER_HITOLO,
	    .p_in_channel = &in_sig2_channel,
	};

	input_config.p_trigger_config = &trigger_config_sig2;

	err = nrfx_gpiote_input_configure(&gpiote, INPUT_SIG_2, &input_config);
	if (err != NRFX_SUCCESS) {
		LOG_ERR("nrfx_gpiote_input_configure error: 0x%08X", err);
		return;
	}

	nrfx_gpiote_trigger_enable(&gpiote, INPUT_SIG_1, true);
	nrfx_gpiote_trigger_enable(&gpiote, INPUT_SIG_2, true);
}


int main(void)
{
	LOG_INF("Hello World! %s\n", CONFIG_BOARD);
	
	gpiote_init();

	return 0;
}
